------------------------------
--      Are you local?      --
------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Necronomicon")
local BS = AceLibrary("Babble-Spell-2.2")

---------------------------------
--      Addon Declaration      --
---------------------------------

Necronomicon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0")

Necronomicon.revision = tonumber(string.sub("$Revision: 27266 $", 12, -3))

Necronomicon:SetModuleMixins("AceEvent-2.0")

Necronomicon.defaults = {
	tooltips = true,
}

Necronomicon.consoleOptions = {
	type = "group",
	handler = Necronomicon,
	args = {
	},
}

Necronomicon:RegisterDB("NecronomiconDB", "NecronomiconDBPerChar")
Necronomicon:RegisterDefaults("profile", Necronomicon.defaults)
Necronomicon:RegisterChatCommand({ "/Necronomicon" }, Necronomicon.consoleOptions, "NECRONOMICON")

------------------------------
--      Initialization      --
------------------------------

function Necronomicon:OnInitialize()
	self.hasspell = {}
	self.spelllist = {}
	self.itemlist = {}
	self.itemcount = {}
	if not self.version then self.version = GetAddOnMetadata("Necronomicon", "Version") end
	local rev, name = self.revision, nil
	for name, module in self:IterateModules() do
		rev = math.max(rev, module.revision)
	end
	self.version = (self.version or "1.0") .. " |cffff8888r" .. rev .. "|r"
end

function Necronomicon:OnEnable()
	local class, enclass = UnitClass("player")
	if enclass == "WARLOCK" then
		if AceLibrary("AceEvent-2.0"):IsFullyInitialized() then
			self:AceEvent_FullyInitialized()
		else
			self:RegisterEvent("AceEvent_FullyInitialized")
		end
	else
		DisableAddOn("Necronomicon2")
	end
end

function Necronomicon:OnDisable()
end

-------------------------------
--      Module Handling      --
-------------------------------

function Necronomicon:RegisterModule(name, module)
	if module:IsRegistered() then
		error(string.format("%q is already registered.", name))
		return
	end

	if module.db and module.RegisterDefaults and type(module.RegisterDefaults) == "function" then
		module:RegisterDefaults("profile", module.defaults or {})
	else
		self:RegisterDefaults(name, "profile", module.defaults or {})
	end

	if not module.db then module.db = self:AcquireDBNamespace(name) end

	
	local m = module.consoleCmd or name

	self.consoleOptions.args[m] = {
		type = "group",
		name = m,
		desc = L["Options for %s."]:format(m),
		args = {
			[L["Active"]] = {
				type = "toggle",
				name = L["Active"],
				order = 1,
				desc = L["Activate or deactivate this module."],
				get = function()
					return not module.db.profile.inactive
				end,
				set = function(v)
					module.db.profile.inactive = not v
					module.core:ToggleModuleActive(m, v)
				end,
			}
		}
	}


	if module.consoleOptions then
		-- if the consoleoption already exists we merge in the data otherwise we create a new option
		if self.consoleOptions.args[m] then
			for k,v in pairs(module.consoleOptions.args) do self.consoleOptions.args[m].args[k] = v end
		else
			self.consoleOptions.args[m] = module.consoleOptions
		end
	end

	module.registered = true
	if module.OnRegister and type(module.OnRegister) == "function" then
		module:OnRegister()
	end
end

---------------------------------
--       Event Handlers        --
---------------------------------

function Necronomicon:AceEvent_FullyInitialized()
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")

	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterBucketEvent("BAG_UPDATE", 0.5, "ScanBags")

	local name, module = nil, nil
	for name, module in self:IterateModules() do
		if module.db.profile.inactive then 
			self:ToggleModuleActive(module, false)
		else
			self:ToggleModuleActive(module, true)
		end
	end

	-- first learn all spells, then scan bags. Do not change order.
	self:LEARNED_SPELL_IN_TAB()
	self:ScanBags()
end

function Necronomicon:PLAYER_REGEN_ENABLED()
	-- issue out of combat updates
end

function Necronomicon:PLAYER_REGEN_DISABLED()
	-- issue in combat lockdown
end

function Necronomicon:LEARNED_SPELL_IN_TAB()
	local i = 1
	local enspell, spell, subspell

	self.hasspell = {}

	while true do
		spell, subspell = GetSpellName(i, BOOKTYPE_SPELL)
		if not spell then
			do break end
		end
		enspell = BS:HasReverseTranslation( spell ) and BS:GetReverseTranslation( spell ) or nil
		if enspell and self.spelllist[enspell] then
			self.hasspell[enspell] = { spell = spell, subspell = subspell, id = i }
		end
		i = i + 1
	end
	self:TriggerEvent("Necronomicon_SpellsUpdated")
end

function Necronomicon:ScanBags()
	local itemid
	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			_,_,itemid = string.find(GetContainerItemLink(bag,slot) or "", "item:(%d+):%d+:%d+:%d+")
			itemid = tonumber(itemid)
			if itemid and self.itemlist[itemid] then
				self.itemcount[itemid].bag = bag
				self.itemcount[itemid].slot = slot
				self.itemcount[itemid].equipped = nil
			end
		end
	end
	_,_, itemid = string.find( GetInventoryItemLink("player",GetInventorySlotInfo("RangedSlot") or ""), "item:(%d+):%d+:%d+:%d+")
	itemid = tonumber(itemid)
	if itemid and self.itemlist[itemid] then
		self.itemcount[itemid].bag = nil
		self.itemcount[itemid].slot = nil
		self.itemcount[itemid].equipped = true
	end
	local count
	for k,v in pairs(self.itemlist) do
		count = GetItemCount( k )
		if count ~= self.itemcount[k] then
			self.itemcount[k].count = count
		end
	end
	self:TriggerEvent("Necronomicon_BagUpdate")
end
